import { useEditor } from "@/modules/editor";
import { defineComponent } from "vue";
import { string } from "vue-types";
import { useCompData } from ".";
import { View } from "../View";

export const Component = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props) {
    const comp = useCompData(props.compId);

    const { store, controls, helper } = useEditor();

    async function changeVal() {
      try {
        const url = await controls.pickCtrl.pickOneImage();
        if (!url) return;

        comp.value.url = url;
        comp.value.x = 0;
        comp.value.y = 0;
        comp.value.s = 1;
      } catch (error) {
        console.log(error);
      }
    }

    return () => {
      const value = comp.value;
      const scale = value?.s || 1;
      const ox = value?.x || 0;
      const oy = value?.y || 0;
      const objectFit =
        scale + "" == "1" && ox + "" == "0" && oy + "" == "0"
          ? "cover"
          : "contain";

      return (
        <View
          class="overflow-hidden"
          compId={props.compId}
          onDblclick={store.isEditMode ? changeVal : undefined}
        >
          <img
            crossorigin="anonymous"
            class={"w-1/1 h-1/1 object-cover pointer-events-none"}
            style={{
              transform: `scale(${scale}) translate(${ox}%,${oy}%)`,
              objectFit,
            }}
            src={
              value.url.startsWith("data:image/png")
                ? value.url
                : value.url + "?editMode=" + store.isEditMode
            }
            onLoad={() => {
              if (helper.isCurrComp(props.compId)) {
                controls.transferCtrl.initStyle();
              }
            }}
          />
        </View>
      );
    };
  },
});
