import { IconAdd, IconMove } from "@/assets/icons";
import { css } from "@linaria/core";
import { IconDelete } from "@queenjs/icons";
import { defineComponent } from "vue";
import { string } from "vue-types";
import { useEditor } from "../../..";
import { useCompRef } from "./hooks";

export const View = defineComponent({
  props: {
    compId: string().isRequired,
  },
  emits: ["dblclick"],
  setup(props, { slots, emit }) {
    const { store, actions, helper, controls } = useEditor();
    const compRef = useCompRef(props.compId);

    return () => {
      const comp = helper.findComp(props.compId);
      if (!comp) return store.isEditMode ? <div>无效组件</div> : null;
      const isStreamCard = helper.isStreamCard(props.compId);
      const isGroupComp = helper.isGroupComp(props.compId);

      return (
        <div
          ref={compRef}
          class={[
            viewStyle,
            store.isEditMode && editCompStyle,
            isGroupComp && groupCompCls,
          ]}
          style={helper.createStyle(comp.layout)}
          // onClick={(e) => {
          //   e.stopPropagation();
          //   if (store.isEditMode) {
          //     actions.pickComp(props.compId);
          //   }
          // }}
          onDblclick={() => emit("dblclick")}
        >
          <div
            // onMousedown={(e) => {
            //   if (helper.isGroupCompChild(props.compId)) return;

            //   // e.stopPropagation();
            //   if (store.isEditMode) {
            //     actions.pickComp(props.compId);
            //     if (!helper.isStreamCard(props.compId)) {
            //       controls.transferCtrl.mousedown(e, "move", store.currComp);
            //     }
            //   }
            // }}
            onMousemove={(e) => {
              if (
                !store.isEditMode ||
                !controls.dragAddCtrl.dragingCompKey ||
                !helper.isStreamCard(props.compId)
              )
                return;
              actions.pickComp(props.compId);
            }}
          >
            {slots.default?.()}
          </div>

          {store.isEditMode && isStreamCard && (
            <div class="hudop">
              {store.streamCardIds.length > 1 && (
                <IconMove class="draganchor" />
              )}
              {store.streamCardIds.length > 1 && (
                <IconDelete
                  onClick={() => actions.removeStreamCard(props.compId)}
                />
              )}
              <IconAdd
                onClick={() => {
                  const index = store.streamCardIds.indexOf(props.compId) + 1;
                  actions.addCompToDesign("Container", index);
                }}
              />
            </div>
          )}
        </div>
      );
    };
  },
});

const viewStyle = css`
  position: relative;
  font-size: 0;
  cursor: pointer;

  > :first-child {
    width: 100%;
    height: 100%;
  }

  .hudop {
    position: absolute;
    top: 0px;
    left: -46px;
    background-color: white;
    flex-direction: column;
    color: black;
    display: flex;
    font-size: 12px;
    width: 28px;
    align-items: center;
    border-radius: 4px;
    .inficon {
      padding: 8px;
    }
    z-index: 1000;
  }
`;

const editCompStyle = css`
  &:hover {
    outline: 2px dashed @inf-primary-color;
  }
`;

const groupCompCls = css`
  outline: 2px solid @inf-primary-color !important;
`;
