import { AnyFun } from "queenjs/typing";
import { EditorModule } from "../../module";
import { Action, HistoryController } from "./HistoryController";

export class HistoryCtrl {
  history: HistoryController;
  historyActionDoing = false;

  constructor(protected module: EditorModule, historyTotal = 50) {
    this.history = new HistoryController();
    this.history.state.maxLen = historyTotal;
  }

  record(
    root: any,
    type: Action["type"],
    paths: string[],
    value: any,
    oldValue: any
  ) {
    if (this.historyActionDoing) {
      const action = new Action(type, root, paths.join("."), value, oldValue);
      this.history.record(action);
    }
  }

  bindActions(actNames: string[]) {
    const actions: any = this.module.actions;
    actNames.forEach((actName) => {
      const action = actions[actName];
      actions[actName] = this.proxyAction.bind(this, action);
    });
  }

  async proxyAction(action: AnyFun, ...args: any[]) {
    const { history } = this;
    if (this.historyActionDoing) {
      return await action(...args);
    }
    try {
      this.historyActionDoing = true;
      await action(...args);
      history.submit();
    } catch (error) {
      console.error(error);
      history.cacheGroupAction.undo();
      history.cacheGroupAction.actions = [];
    } finally {
      this.historyActionDoing = false;
    }
  }
}
